function [acc,dt,ntot] = parseAT2_all(thfilename)
% Function parseAT2_all
%-------------------------------------
% DESCRIPTION:
% the function is a part of get_recinfo (by Vamvatsikos) used only for
% PEER-SILVA records

%-------------------------------------
% INPUT:
%-------------------------------------
%  thfilename : the time history file name, that we want to investigate
%               note that it must be the full path and name for us to find
%               it''s location.
%-------------------------------------
% OUTPUT:
%-------------------------------------
% acc,dt,ntot: Records info (Accelleration, sampling time and number of
% points)

% below follows what a header (or tail) of Silva record looks like
%----------------------------------------------------
%PACIFIC ENGINEERING AND ANALYSIS STRONG-MOTION DATA
%LOMA PRIETA 10/18/89 00:05, HOLLISTER DIFF ARRAY, 165
%ACCELERATION TIME HISTORY IN UNITS OF G
%NPTS=  7928, DT= .00500 SEC
%-----------------------------------------------------
%      The header in the PEER NGA record is formatted as follows:
%------------------------------------------------------
%PEER NGA STRONG MOTION DATABASE RECORD
%CORINTH EQ, 02/24/81, M6.7, CORINTH, TRANSVERSE                                 
%ACCELERATION TIME HISTORY IN UNITS OF G
%4094    0.0100    NPTS, DT
%------------------------------------------------
% OR
%------------------------------------------------
%   PEER NGA STRONG MOTION DATABASE RECORD
%   FRIULI, ITALY 05/06/76 2000, BARCIS, 000                                        
%   ACCELERATION TIME HISTORY IN UNITS OF G
%   NPTS=3309,    DT=0.0050SEC
%-------------------------------------------------
    % if this is a header move it to the tail, otherwise neither snap, nor
    % drain2dx, nor snap2dx will be able to read it
    % NOTE: We could use "m" to get the number of rows, instead 
    % of reading NPTS, but we would never be able to figure out 
    % "DT". So we need the header.
    acc=[];
    fid=fopen(thfilename,'r');
    m=0;   % a line counter
    ngaflag=0;
    fline=cell(4,1);
    headerfound=0;
    while 1
      m=m+1;
      % keep 4 lines, the current plus three past. The current minus 2 positions
      % containts the title when you hit the NPTS line.
      
      fline=fgetl(fid);
      % ofline works like a stack of size 4
      k=mod(m-1,4)+1;
      ofline{k}=fline;
      
      % find out if this is a PEER NGA file. Do this by looking for PEER NGA
      % at the start of any line. For speed, perform
      % this comparison only if the ngaflag is still zero!
      
      % get to the point where they start providing NPTS! 
      if ~headerfound
         if strncmp(strtok(fline),'NPTS=',5)
            [ntot,dt]=strread(fline,'NPTS= %d , DT= %f',1);
            rectitle=ofline{mod(m-3,4)+1};
            % renumber to get the right order for the header.
            %header=ofline(mod(m-[4,3,2,1],4)+1);
            headerfound=1;
            headerpos=m;
            continue
         elseif strfind(fline,'NPTS')>5
            % "NPTS" must appear beyond the first 5 characters
            [ntot,dt]=strread(fline,'%d %f',1); %use pattern only once, otherwise it fails!
            rectitle=ofline{mod(m-3,4)+1};
            % renumber to get the right order for the header.
            %header=ofline(mod(m-[4,3,2,1],4)+1);
            headerfound=1;
            headerpos=m;
            continue
         end
      end 
         
      if ~ischar(fline)   % this is a nice way to check for end-of-file
         if ~headerfound
            Eflag=1;
            pwd_opening=getappdata(0,'pwd_opening');
            if isempty(pwd_opening)==0
                cd(pwd_opening)
            end
            errordlg('Not a valid PEER/Silva record','Error')
            error('Not a valid PEER/Silva record')
         else
            break
         end
      else
         [accline,count,errmsg]=sscanf(fline,'%f');
         if ~isempty(accline) % make sure it contains numbers!
            acc=[acc;accline];
         end
      end
    end
    
    if isempty(dt)||isempty(ntot), Eflag=1; end
    ncol=5; % standard Silva 5-columns!
    units=981; %(convert from g's to cm/s2)
    nrow = (ntot - mod(ntot,ncol))/ncol;
    % For files that have the text info as header, move it to 
    % the back
    fclose(fid);
   
    if headerpos<=4  
       fprintf('\n Virgin Rec, moving header to the tail\n')
       fid=fopen(thfilename,'r');
       for i=1:headerpos
          % I know I am reading it again, but I need to do it anyway to discard
          % the first four lines of the header and go to accel values.
          header{i}=fgets(fid);
       end
       tmp1=tempname;
       fout=fopen(tmp1,'w');
       while 1
          fline = fgets(fid);
          if ~ischar(fline), break; end
          fprintf(fout,'%s',fline);
       end
       % put the header at the end
       for i=1:headerpos
          fprintf(fout,'%s',header{i});
       end
       fclose(fout)
       fclose(fid)
       % remove original file and replace it with the new one
       copyfile(tmp1,thfilename);
       delete(tmp1);
    end
t=linspace(0,(ntot-1)*dt,ntot);
if  length(acc)<length(t);
    acc=[acc' acc(end)]';
end
if length(acc)~=length(t)
    acc=acc(1:length(t));
end

    