function [dr,cv,cvsa,lowest_C,sval,trunc_dv,trunc_sv,tdvalues,tsvalues]=...
         postIDA_interpolateIDA(svalues,...
			     dvalues,sa_range,fitcode,plotflag,dr_range,trunc_val,relaxNaNs)

% function [sa_range,dr,cv,cvsa,maxsa,last_sa]=...
%    postIDA_interpolateIDA(svalues,dvalues,sa_range,fitcode,plotflag);
%
% This routine interpolates the IDAs and then generates stripes of
% dvalues for given svalues levels.
%__________________________________________________________________
% INPUT 
% svalues,dvalues : cell arrays 1xN of vectors of Sas and drifts
% sa_range        : a SORTED vector of Sa-levels to compute drift
%                   stripes. If empty, no stripes are generated
% fitcode         :  0 = piecewise linear fit
%                    1 = std. natural splines (cscvn2.m)
%                    2 = adv. natural splines (fitIDA.m)
% trunc_val       : a single EDP value to truncate the IDA or 
%                   N IM-values to truncate each IDA separately
% relaxNaNs       : if 0 (default) then the first NaN signals collapse.
%                   This is good for robust models, where we are certain that
%                   convergence is easy to achieve.
%                   if 1 (careful!) then if there exists a convergent run
%                   with drift less than Th_drift directly above an early NaN, 
%                   then that NaN is considered bogus (numerical non-convergence but
%                   not dynamic instability) and it is dropped! So instead
%                   of calling it a "resurrection", we decide that we had
%                   no collapse in the first case. 
%                   if 2 (VERY careful!) even two or more consecutive NaNs could be 
%                   indeed considered collapse if at least one convergent
%                   run (with drift<Th_drift) exists above them. This is
%                   for very shaky models only.
%
% OUTPUT 
% dr              : array N*length(sa_range), each column is one
%                   stripe at fixed Sa-level. Each row corresponds
%                   to one record.
% cv              :  use cv{.} to plot spline fits (x=drift)
% cvsa            :  use cvsa{.} to compute drifts given SAs (x=Sa)
% lowest_C        : array Nx1, of indices of the lowest (in Sa)
%                   convergent points. One for each IDA. This is a
%                   by-product of the filtering done to do the
%                   interpolation.
%                   NOTE THAT the value of lowest_C may change if you
%                   specify a non-zero value for relaxNaNs. This may make
%                   some intermediate NaNs to be removed and lowest_C will
%                   thus become higher.
%
% trunc_dv,trunc_sv : the dv and sv values where truncation has occured
% tsvalues,tdvalues : the svalues and dvalues corresponding to the truncated
%                     IDAs that are returned in cv.
%___________________________________________________________________    
% 
% Copyright (c) 1999 by D.Vamvatsikos
% Updated Sep 18, 2001: Must be upgraded SOON to deal with structural
%                       resurrection. Have added capability to produce SA-values
%                       given a dr_range
% Updated Dec 13, 2004: Made compatible with new flipcscvn which works with
%                       cells
% Updated Feb 19, 2005 : Now it supports truncation of IDAs. It  reads input
%                        trunc_val and outputs trunc_sv,trunc_dv
%                        for truncated IDAs! All the outputs refer to the
%                        truncated curves!
% Updated  8/May/2006: relaxNaNs parameter added to properly handle non-robust 
%                      models that may show early collapses due to
%                      non-convergence while convergent runs clearly exist
%                      at higher IMs. This is for LA9_2Dmfrag that shows
%                      this behavior, with clear indications that it is not
%                      a resurrection!
% Updated 3/Apr/2008: Fixed an issue with relaxNaNs where having no NaNs in
%         a given curve would cause lowest_C to return as 0!!
% Updated 12/Jun/2010: Fixed an issue where supplying a "dr_range"
%         containing EDP values higher than the trunc_val in EDP would
%         cause a "step" in the resulting "sval". The reason is that I was
%         replacing all Savalues above for drift beyond trunc_val by the
%         flatline Sa of the original IDA, instead of the truncated one.

% default behavior is to keep all NaNs as strict collapses.
if nargin<8, relaxNaNs=0; end
if nargin<7, trunc_val=[]; end
if nargin<6, dr_range=[]; end
% default is no plotting
if nargin<5,  plotflag=0; end
% default is cscvn2.m fitting of splines
if nargin<4, fitcode=1; end
% default is no stripe-generation
if nargin<3, sa_range=[]; end

% sort just in case, as x2y_s requires sorted input!
sa_range=sort(sa_range);

maxR=length(svalues);


%if any(isnan(cat(1,dvalues{:}))), error('can''t handle NaNs yet');end

% Newer versions of traceDPOX (X>4) include all the Infs, NaNs into
% the dvalues,svalues. For now we will simply exclude these points
% from further consideration. Still, this HIDES all intermediate
% collapses, so be careful, you may be missing something!


%%% 05/10/2017: fissa i vettori EDP e IM di partenza, potrebbero servire in
%%% caso di problemi con il troncamento
dvalues_unmod=dvalues;
svalues_unmod=svalues;



for i=1:maxR
  if relaxNaNs==0 
     % filter out all points above the lowest (in SA) NaN or Inf. So
     % keep points up to the first flatline only!!
     % svalues must end at the first NaN (or Inf) point, or at the last
     % available point (if convergent). So find the index of the
     % lowest convergent point! This will exclude ressurrections!
     lowest_C(i)=min([find(isnan(dvalues{i})|isinf(dvalues{i}))',...
                 length(dvalues{i})+1])-1;
     svalues{i}=svalues{i}(1:lowest_C(i));
     dvalues{i}=dvalues{i}(1:lowest_C(i));
  elseif relaxNaNs==1
     allNaNs=find(isnan(dvalues{i})|isinf(dvalues{i}))';
     % initialize lowest_C just in case no NAN is found
     lowest_C(i)=length(dvalues{i});
     for j=1:length(allNaNs)
        if allNaNs(j)<length(svalues{i})&(~isnan(dvalues{i}(allNaNs(j)+1)))&...
                                      (~isinf(dvalues{i}(allNaNs(j)+1)))
           % the next IM is convergent so remove this NaN
           svalues{i}(allNaNs(j))=[];
           dvalues{i}(allNaNs(j))=[];
        end
        % Calculate a new value for lowest_C, and drop all NaNs above
        lowest_C(i)=min([find(isnan(dvalues{i})|isinf(dvalues{i}))',...
                 length(dvalues{i})+1])-1;
        svalues{i}=svalues{i}(1:lowest_C(i));
        dvalues{i}=dvalues{i}(1:lowest_C(i));
     end
  else
     allNaNs=find(isnan(dvalues{i})|isinf(dvalues{i}))';
     % initialize lowest_C just in case no NAN is found
     lowest_C(i)=length(dvalues{i});
     highest_C=max(find((~isnan(dvalues{i}))&(~isinf(dvalues{i})))');
     for j=1:length(allNaNs)
        if allNaNs(j)<highest_C
           % a higher convergent IM-value exists, remove this NaN
           svalues{i}(allNaNs(j))=[];
           dvalues{i}(allNaNs(j))=[];
        end
        % Calculate a new value for lowest_C, and drop all NaNs above
        lowest_C(i)=min([find(isnan(dvalues{i})|isinf(dvalues{i}))',...
                 length(dvalues{i})+1])-1;
        svalues{i}=svalues{i}(1:lowest_C(i));
        dvalues{i}=dvalues{i}(1:lowest_C(i));
     end
  end
end;

% first make all the NaN drifts into Infs (for homogeneity)
% Identify consecutive segments of Infs. If only one (at the end)
% then we have a simple case of collapse. If multiple segments of
% Infs exist (seperated by non-Inf drifts) then we have found an
% intermediate collapses plus the final. How to postprocess those?
% Remove all the Infs. Between each Inf and non-Inf place (say)
% 2-points. One at quarter-length, another at half-length
% (Sa-wise). Then just calculate some appropriate drifts. This is
% tough. you need to do some exponential fit or assume some
% parameters right there. Maybe get the slope to the previous
% non-inf points (if it exists) and make it doubly steeper etc. 
% if no such slope exists (e.g. a non-Inf point between 2 infs)
% just use some ad-hoc rule (double the drift of the non-Inf point?)
% REMEMBER to replace the svalues,dvalues that return to the
% program and NEVER save them on the same file. These aren't real runs.


if plotflag
  Hf_wait=waitbar(0,'Spline fitting and digitizing...');
end  

%// interpolate all splines in the current suite
for R=1:maxR
  %// use cv{.} to plot
  switch fitcode
   case 0
       curve_p=zeros(2,length(svalues{R})*2);
       curve_p(:,1:2:end)=[dvalues{R},svalues{R}]';
       curve_p(:,2:2:end)=[dvalues{R},svalues{R}]';
       cv{R}=cscvn2(curve_p);
    % This is an intelligent way to put piecewise linear models in 
    % spline-like format. This will make things a lot easier for you.
   case 1
       cv{R}=cscvn2([dvalues{R},svalues{R}]');
   case 2
       cv{R}=fitIDA(svalues{R},dvalues{R});
  end
  
  % use cvsa{.} to compute drifts given SAs
  cvsa(R)=flipcscvn(cv(R));
  
  % if a truncation dv or sa has been supplied, apply it!!
  if ~isempty(trunc_val)
     if length(trunc_val)==1
        [cv(R),trunc_sv(R),truncflag]=truncateIDA(cv(R),trunc_val);
        trunc_dv(R)=trunc_val;
        if truncflag
           indy=find(svalues{R}<trunc_sv(R));
           tsvalues{R}=[svalues{R}(indy);trunc_sv(R)];
           tdvalues{R}=[dvalues{R}(indy);trunc_val];
           % adjust lowest_C to point to the truncation point!
           lowest_C(R)=indy(end)+1;
           cvsa(R)=flipcscvn(cv(R));
        else
           tsvalues(R)=svalues(R);
           tdvalues(R)=dvalues(R); % remember these may be different that input!
        end
     elseif length(trunc_val)==maxR
        [cvsa(R),trunc_dv(R),truncflag]=truncateIDA(cvsa(R),trunc_val(R));
        trunc_sv(R)=trunc_val(R);
        if truncflag
           indy=find(svalues{R}<trunc_val(R));
           tsvalues{R}=[svalues{R}(indy);trunc_val(R)];
           tdvalues{R}=[dvalues{R}(indy);trunc_dv(R)];
           % adjust lowest_C to point to the truncation point!
           lowest_C(R)=indy(end)+1;
           cv(R)=flipcscvn(cvsa(R));
        else
           tsvalues(R)=svalues(R);
           tdvalues(R)=dvalues(R); % remember these may be different that input!
        end
     else 
        error('incorrect trunc_val')
     end
     
  else
     trunc_dv=[];trunc_sv=[];
     tsvalues(R)=svalues(R);
     tdvalues(R)=dvalues(R);
  end   
  
  %// use the faster x2y_s() to get drifts for each IDA curve
  if ~isempty(sa_range)
    dr(R,:)=x2y_s(sa_range,cvsa{R});
  else
    dr=[];
  end
  
  % similarly compute fractile svalues given dvalues. 
  if ~isempty(dr_range)
    sval(R,:)=x2y_s(dr_range,cv{R});
    % but since the cv{R} usually doesn't include the flatline, 
    % we need to simulate its effect in sval, otherwise a lot of
    % NaNs get produced (due to attept to extrapolate) when there
    % should be none (there is always some finite IM to produce a 
    % given DM). So fix the NaNs by replacing them, for each
    % record, with the records' flatline Sa. I wonder if I should
    % use a better estimate of the flatline capacity here (like the 
    % average of highest C with lowest NC point), or would this
    % causes a discrepancy? 
    i_sval=find(isnan(sval(R,:)));
    % since we have removed non-convergent sa-values, the line
    % below works 
    %sval(R,i_sval)=svalues{R}(end); % Bad results if you have truncated. 
    sval(R,i_sval)=tsvalues{R}(end); % Works every time! 12/Jun/2010
  else
    sval=[];
  end
  
  if plotflag
    waitbar(R/maxR);
  end  
end
%%% 05/10/2017: controlla se ci sono valori pari a 0 in tdvalues (vuol dire
%%% che c' stato un problema (probabilmente il valore di troncamento  >
%%% di tutti i valori del vettore dvalues:
for R=1:maxR
    if isequal(tdvalues{i},0) | isnan(tdvalues{i}(2))
         tsvalues(R)=svalues_unmod(R);
         tdvalues(R)=dvalues_unmod(R);
    end
end
if plotflag
  close(Hf_wait);
end
