function [x]=quantile(X,q)
% function [x]=quantile(X,q)
% Works along the columns. For example, quantile(X,0.5)==median(X)
%-------------------------------------------------------------------
% produces medians and log(std's) for Pushovers.
% INPUT
%   X : matrix NxM of samples
%   q : vector Kx1 of quantiles
% OUTPUT
%   x : matrix KxM  X-values that correspond to the 
%       given quantiles. Column-i contains the quantiles
%       of column-i of X
%-------------------------------------------------------------------
% NOTE : This function produces identical results with the quantile
% of Splus and also with Excel. On the other hand, the "prctile" 
% provided by MatLab Stats toolbox, is quite different in general.
% EXAMPLE: quantile([0,1,2,3,9,10,11,12]',.55) --> 8.1 (as in Splus)
%          prctile([0,1,2,3,9,10,11,12],55)    --> 8.4 (Matlab)
% CAUTION : This will not check for q(i) in [0,1]. 
% This is the user's responsibility!
%
% USE : iquantile when your data X contains NaNs.
% Updated June 15th, 1999
% Updated Feb 14th, 2003
%    made some changes to increase speed and make the routine work even
%    for q=1.
% Copyright (c) 1999 by D.Vamvatsikos

X=sort(X);
[N,M]=size(X);
r = 1 + (N-1)*q;
% calculate index of corresponding quantile (+1 as 1st index is 1 not 0)
i=floor(r);
% Calculate decimal fraction left out.
frac=r-i;
% Now if frac=0, then x=X(i,:), else interpolate linearly between 
% X(i,:) and X(i+1,:). 
X(end+1,:)=X(end);
x=X(i,:)+(X(i+1,:)-X(i,:)).*repmat(frac,1,M);

