function [cSa,cdrift,Kelastic]=setTcap5(svalues,dvalues,curves,frac,Th_drift,mode,plotflag,debugflag,H_graph);
% [cSa,cdrift,Kelastic]=setTcap5(svalues,dvalues,curves,frac,Th_drift,mode);
% driver routine for truecap5.m
%--------------------------------------------------------------------
% INPUT
% svalues : Cell array 1xN of SA values (each entry a column vector)
% dvalues : Cell array 1xN of drift values (each entry a column vector)
% curves  : (optional) a cell array, each entry "i"  containing an 
%           interpolated curve, fit to the points (Sa{i},drift{i})
%           see truecap5.m for more details.
% frac    : the fraction (usually 20%) of elastic stiffness  
%           used for capacity slope (default)
% Th_drift: Threshold drift, the maximum allowable drift capacity value
%           usually set to 10% (default)
% mode    : Determines the way to evaluate capacitities
%           1 : use standard cap-definition & use the Kel of each rec.(default)
%           2 : use standard cap-definition & use the median Kel of all rec.
%           3 : use robust cap-definition & use the Kel of each rec.
%           4 : use robust cap-definition & use the median Kel of all rec.
% plotflag : if 1 then box plots of all capacities are printed (on
%            current figure window)
% debugflag: forces additional output for debugging if 1.
%
% OUTPUT
% Kelastic: a column vector, each entry contains the elastic stiffness of
%           the corresponding DPO
% {medK    : the median Kelastic.}
%
%     this fuction will return 2 vectors, cSa and cdrift containing
%     the capacity (x,y) points for all records in the set
%     of (svalues,dvalues) provided. 
% --------------------------------------------------------------------
% 
% (Last Updated: August 24th, 2000)
% Copyright (c) 1999 by D.Vamvatsikos

maxR=size(dvalues,2);
%// Check Input arguments, enforce default options if neede
if nargin<2 error('too few arguments'); end

if nargin<3
   curves=cell(1,maxR); % force a fit!
else 
   if isempty(curves)
     curves=cell(1,maxR); % force a fit!
   end
end


if nargin<4
   frac=0.2; %// default is 1/5 of elastic!
end
   
if nargin<5 Th_drift=0.1; end

if nargin<6 mode=1; end

if nargin<7 plotflag=1; end   %// default : show boxplots
if nargin<8 debugflag=0; end  %// by default : no debug.
if nargin<9 H_graph=2; end %// use a window other than "1" for plotting

%// set robust or standard fitting
if mode==1|mode==2 
  crit=0;
else 
  if mode==3|mode==4
    crit=1;
  else
    error('unknown mode');
  end
end

%// assume that DPO is dense enough that first point, after (0,0) is
%// elastic! This is going to decide your elastic stiffness
%// unfortunately most DPO's aren't like that
%// by the way my definition is Sa/drift as "stiffness"

if debugflag
  format compact
  fprintf('\n Records to Analyze \n')
end  
slopepoints=0;thpoints=0;endpoints=0;

for R=1:maxR  
  % for LA20 this may cause some warnings of "divide by zero"
  % as 2 stories are unmovable basements, so trying to get story
  % by story capacities causes a 0/0 division here.
  Kelastic(R)=svalues{R}(2)/dvalues{R}(2);
end

medK=median(Kelastic);

if plotflag
  Hf_wait=waitbar(0,'Calculating capacities...');
end  
for R=1:maxR
  if debugflag
    fprintf(': %d ',R)
  end  
  xy=[svalues{R},dvalues{R}];
  if mode==1|mode==3
    [cSa(R),cdrift(R),flag]=truecap5(xy,curves{R},Kelastic(R),frac, ...
                                     Th_drift,crit,debugflag);
  else
    [cSa(R),cdrift(R),flag]=truecap5(xy,curves{R},medK,frac,Th_drift, ...
                                     crit,debugflag);
  end
    
  switch flag
    case 1, slopepoints=slopepoints+1;
    case 2, thpoints=thpoints+1;
    case 3, endpoints=endpoints+1;
    otherwise error('Unknown flag value returned')
  end
  if plotflag
    waitbar(R/maxR);
  end  
end
if debugflag
  fprintf('\n*Analysis done*\n\n')
  fprintf(' Slopepoints: %2d\n Th_drift points %2d\n End points %2d\n',...
          slopepoints,thpoints,endpoints)
  format loose
end

if plotflag
  close(Hf_wait);
end

if plotflag==1
  H_current=gcf;
  figure(H_graph);
  clf;
  subplot(1,3,1)

  % Safeguard against the case of having only 1 unique value. You can''t have
  % a boxplot then! You need at least 2 unique values for that

  if size(unique(cSa),2)>1 
    boxplot(cSa)
    grid off
    title('boxplot of Sa Capacity')
  else
    plot(cSa,'bo') 
  end

  subplot(1,3,2) 

  if size(unique(cdrift),2)>1
    boxplot(cdrift)
    grid off
    title('boxplot of drift Capacity') 
  else 
    title('plot of drift Capacity')
    plot(cdrift,'bo')
  end

  subplot(1,3,3)
  title('Sa vs drift capacity')
  plot(cdrift,cSa,'rs') 

  %// Pause for 1 second
 % pause(1)
  %// return to previously selected figure 
  %// so as not to disturb subsequent plotting by calling program
  figure(H_current)
end