%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parser for ESM C.ASC.ACC files
% ---------------------------------
% Input : Full path of the file to be parsed
% 
% Outputs: 
%   -Acceleration time history (Acc)
%   -Time step used for recording (record_dt)        
%   -Number of recorded points (NPTS)
%   -Mangitude (Mag) - moment magnitude (local if Mw unavailable)
%   -Azimuth of component (azim) >>provisional<<
%   -Record identifier (id)
%   -Focal mechanism (fm)
%   -Epicentral distance in km (Repi)
%   -Site classification per EN-1998 (site)
%   - Consistency indicator (consist)
%       --consist = 1 if all checks pass, -1 if at least one check fails
%   -error code to indicate if the file was not present (errCode)
%       --errCode = 0 if successful, -1 if File not found
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




function [Acc,record_dt,NPTS] = parseESM(filename)


% clear all
% filename='waveforms\E.BUI..HNE.D.19760911.163110.C.ACC.ASC';

file_in = fopen(filename, 'r');
if(file_in == -1)
    Acc = -1;
    record_dt = -1;
    errCode = -1;
    NPTS = -1;
    azim =-1;
    Mag =-1;
    consist = -1;
    id = -1;
    fm = -1;
    Repi = -1;
    site = -1;
else % if there is no error skip 28 lines
    consist=1;
    
    % read 3 lines to get event data
    id1 = fgetl(file_in);
    id2 = fgetl(file_in);
    id3 = fgetl(file_in);
    
    for j=1:5
        ans = fgetl(file_in);
    end
    
    % read moment and local magnitude
    m1 = fgetl(file_in);
    ans = fgetl(file_in);
    m2 = fgetl(file_in);
    
    MW=str2double(m1(14:end));
    ML=str2double(m2(14:end));
    
    if isempty(MW) || isnan(MW)
        Mag=ML;
    elseif isempty(MW) && isempty(ML) 
        Mag=-1;
    else
        Mag=MW;
    end
    

        ans = fgetl(file_in);

    
   % 13th line focal mechanism
    a8=fgetl(file_in);
    fm=a8(17:end);
    
    ans = fgetl(file_in);
   % 15th line Station code
    id4=fgetl(file_in);
    
    for j=1:7
        ans = fgetl(file_in);
    end
    

    % read 23th line to get site classification
    a7=fgetl(file_in);
    site=a7(25:end);
    ans = fgetl(file_in);
    % read 25th line to get epicentral distance
    a6=fgetl(file_in);
    Repi=str2double(a6(24:end));
    for j=1:3
        ans = fgetl(file_in);
    end
    % read 29th line to get sampling time interval
    a1=fgetl(file_in);
    record_dt=str2double(a1(22:end));
    % read 30th line to get sample size
    a2=fgetl(file_in);
    NPTS=str2double(a2(7:end));
    % read 31st line (duration)
    a3=fgetl(file_in);
    % read 32nd line STREAM: HNE - azimuth suspended for now
    a4=fgetl(file_in);
    % read 33rd line UNITS: cm/s^2
    a5=fgetl(file_in);
% skip 31 more lines to get to the acceleration part

    for j=1:31
        ans = fgetl(file_in);
    end
    
[Acc, np] = fscanf(file_in, '%f');
id=strcat(id1(12:end),'_',id3(22:25),'_',id4(15:end),'_',id2(11:end));

if strcmp(a4(end),'N')
    azim=0;
elseif strcmp(a4(end),'E')
    azim=90;
else azim=-1;
end

% Check record consistency with header information!
%--------------------------------------------------
consist=strcmp(a5,'UNITS: cm/s^2');
duration=str2double(a3(12:end));
if NPTS==np && abs(duration-NPTS*record_dt)<record_dt;
    consist=1;
else consist=0;
end 
    
fclose(file_in);
errCode = 0;
end    
