# simpleSDOF.tcl
#
# Simple SDOF file, suitable for static or dynamic analyses.
#-----------------------------------------------------------
# Created 25/Nov/2011, Copyright (c) D. Vamvatsikos
#       Based on my more complex simpleSDOF4.tcl
# Updated 12/Apr/2013, added Elastic and Elastoplastic materials, plus 
#       an absolute acceleration recorder. I have also set runIDAflag=0 in all cases to avoid
#       potential misinterpretations with older output for new users.
#-----------------------------------------------------------
# NOTE: Massproportional damping always works. Stiffness proportional doesn't work if you have
#       only Zerolength elements.
# NOTE: OpenSEES can't do pushovers with SDOFs having negative slope. Even Arclength doesn't converge.
#       The trick I have implemented is to add a parallel elastic spring, thus making sure that my
#       total system always has postive stiffness and both springs have the same imposed displacements.
#       Then I just record only the force of the forces from the degrading spring (that I am interested
#       in after all!).
wipe

# apparently using "\" creates problems, while "/" is fine. This should also work
# just file for UNIX systems so it is much better!!
source [concat [pwd]/util/units_constants_metric.tcl]
source [concat [pwd]/dati.tcl]
source [concat [pwd]/dati_Isteresi.tcl]
source [concat [pwd]/analysistype.tcl]


#-----------------------------
# Select type of analysis
#set analysistype "RCycD"
#set analysistype "eigenvalue"
# set analysistype "dynamic"
# set analysistype "pushover"
#set analysistype "D2SPO"
#set analysistype "D2IDA"

# set Nextracycles to the number of periods that you want to run free vibration for and
# help estimate residual displacements. The lower the damping, the more you would normally need.
# Overall, though, it is more efficient to just run a few and get an average after a few cycles, 
# since the oscillator will always elastically oscillate around the residual in free vibration.
# Obviously you can override the value below in MatIDA_paramfile.tcl
set Nextracycles 5.0

#----------------------------------
# select type of structural damping. 
# CAREFUL: Stiffnessproportional == NO damping for this zerolength element, unless explicitly 
# included with a damper or a "tangent damping" for elastic_with_damping material!
set dampingtype "massproportional"
#set dampingtype "stiffnessproportional"
#-------------------------------------
## Default selection of loading protocol for RCycD analysis. Good for ATC stuff!
#set peakpts [list .005 .005 .01 .01 .02 .02 .03 .03 .04 .04 .05 .05 .06 .06 .07 .07 .08 .08 .09 .09 .1 .1 .2 .2 .3 .3 .4 .4 .5 .5 .6 .6 .7 .7 .8 .8 .9 .9 1.0 1.0]
#--------------------------------------
# Displacement step for the pushover
# good for ATC-stuff
#set DispStep  [expr 1e-4]
# Good for non-ATC stuff
set DispStep  [expr 1e-3*$m]
#------------------------------------
# set material type
#set material "atc62_spring1a"
#set material "atc62_spring1b"
#set material "atc62_spring2a"
#set material "atc62_spring2b"


# if {[file exists "MatIDA_paramfile.tcl"] & $analysistype=="dynamic"} {
   # puts "Using EXTERNAL Parameters data-file (Matlab-generated)"
   source Mat_paramfile.tcl
# } else {
   # puts "Using STANDARD Parameters data-file"
   # source MatCYC_paramfile.tcl
# }

model basic -ndm 1 -ndf 1


#---------------------------------------------------------------------------
# Source the analysistype information. Better to have it in a distinct file 
# especially if running multiple analyses of the same case
# if {[file exists "MatIDA_anlsfile.tcl"]} {
   # puts "Reading EXTERNAL Analysistype file (Matlab-generated)"
   # source MatIDA_anlsfile.tcl
# }


#-------------------------------------------------------------------------------
puts "Analysis: $analysistype"

#-------------------------------------------------------------------------
# Source the timehistory information to apply to the bridge. The default
# file applies "HDA255.AT2.txt" and "HDA165.AT2.txt", upscaled by 7.5
# if {$analysistype=="dynamic" & [file exists "MatIDA_thfile.tcl"]} {
   # set runIDAflag 0
   # puts "Using EXTERNAL Timehistory data-file (Matlab-generated)"
   # source MatIDA_thfile.tcl
# } else {
   # set runIDAflag 0
   # puts "Using STANDARD Timehistory data-file"
   # puts $analysistype
   # puts [pwd]
   # # source MatIDA_thfile.tcl
# }
if {$analysistype=="dynamic" & [file exists [concat [pwd]/MatIDA_thfile.tcl]]} {
   set runIDAflag 1
   puts "Using EXTERNAL Timehistory data-file (Matlab-generated)"
   source MatIDA_thfile.tcl
} elseif {$analysistype=="D2SPO"} {
   set runIDAflag 1
   puts "Using EXTERNAL Timehistory data-file (Matlab-generated)"
   source MatIDA_thfile.tcl
} elseif {$analysistype=="dynamic"} {
   set runIDAflag 0
   puts "Using STANDARD Timehistory data-file"
   source standard_thfile.tcl
   } elseif {$analysistype=="D2IDA"} {
   set runIDAflag 1
   puts "Using EXTERNAL Timehistory data-file (Matlab-generated)"
   source MatIDA_thfile.tcl
} else {
   set runIDAflag 0
}



#-------------------------------------------------------------------------
# max allowed subdivisions of dt. Use 1 for fast runs, but I highly suggest using 2 for better
# accuracy. It allows the structure to go to higher flatlines (at least in a few cases I have tried).
set dtsub 1.0
# number of iterations
set niter 10

#-------------------------------------------------------------------------------
# Define the material to test.
#
puts materiale=$material
set i 1
set ai 1.0

 ########################################################################################################
 ########################################################################################################
 # Definisci parametri. I parametri così definiti possono essere usati direttamente per il pinching4 mentre vanno modificati per IMKmodified.
 #
  
 		set My [expr $My]
		#puts My=$My
		set My_neg [expr $My_neg]
		#puts My_neg=$My_neg
		set fi_y [expr $fi_y]
		#puts fi_y=$fi_y
		set ah [expr $ah]
		#puts ah=$ah
		set ac [expr $ac]
		#puts ac=$ac
		set mu_c [expr $mu_c]
		#puts mu_c=$mu_c
		set mu_f [expr $mu_f]
		#puts mu_f=$mu_f
		set rp [expr $rp]
		#puts rp=$rp
		# set rDispP [expr $rDispP]
		# puts rDispP=$rDispP
		# set fForceP [expr $fForceP]
		# puts fForceP=$fForceP
		# set rDispN [expr $rDispP]
		# puts rDispN=$rDispN
		# set fForceN [expr $fForceP]
		# puts fForceN=$fForceN



		#uniaxialMaterial ElasticPP 55 $k1  $dy
		#uniaxialMaterial Steel01 55 $Fy $k $b1



		set ePf1 $My
		puts ePf1=$ePf1
		set eNf1 $My_neg
		set ePd1 $fi_y 
		puts ePd1=$ePd1
		set eNd1 [expr $My_neg/($My/$fi_y)]
		puts eNf1=$eNf1
		puts eNd1=$eNd1


		set ePd2 [expr $fi_y*$mu_c] 

		set eNd2 [expr $eNd1*$mu_c]
		set ePf2 [expr $My*(($mu_c-1)*$ah+1)]
		puts ePf2=$ePf2 
		set eNf2 [expr $My_neg*(($mu_c_neg-1)*$ah+1)]
		puts ePd2=$ePd2
		puts eNf2=$eNf2
		puts eNd2=$eNd2
        set K0 [expr $ePf1/$ePd1]
		if {$rp == 0.000000001} {
			if {$ac < 0} {
			set ePf3 [expr +$ac*$K0*($mu_f*$fi_y-$ePd2)+$ePf2]
			set eNf3 [expr +$ac*$K0*($mu_f_neg*$eNd1-$eNd2)+$eNf2]
			} else {
			set ePf3 [expr $My*($ePf2/$My+($mu_f-$mu_c)*$ac)]
			set eNf3 [expr $My_neg*($eNf2/$My_neg+($mu_f_neg-$mu_c_neg)*$ac)]
			}

			set ePd3 [expr $fi_y*$mu_f]
			set eNd3 [expr $eNd1*$mu_f_neg]
			set ePf4 [expr 0.000000001]
			set ePd4 [expr $ePd3] 
			set eNf4 [expr 0.000000001]
			set eNd4 [expr $eNd3] 

		} else {
		set ePf3 [expr $My*$rp]
		set ePd3 [expr $fi_y*((($ePf2/$My-$rp)/$ac)+$mu_c)]
		
		set ePd3 [expr ($ePf2-$rp*$My+$mu_c*$fi_y*(-$ac*$K0))/(-$ac*$K0)]
		set ePf4 [expr $My*$rp]
		set ePd4 [expr $fi_y*$mu_f] 

		set eNf3 [expr $My_neg*$rp]
		set eNd3 [expr $eNd1*((($eNf2/$My_neg-$rp)/$ac)+$mu_c)]
		set eNd3 [expr ($eNf2-$rp*$My_neg+$mu_c_neg*$eNd1*(-$ac*$K0))/(-$ac*$K0)]
		set eNf4 [expr $My_neg*$rp]
		set eNd4 [expr $eNd1*$mu_f_neg] 
		} 
puts ePf3=$ePf3
puts ePd3=$ePd3 
puts eNf3=$eNf3
puts eNd3=$eNd3 
puts ePf4=$ePf4
puts ePd4=$ePd4 
puts eNf4=$eNf4
puts eNd4=$eNd4 
set ePd3_forIMK [expr (-$ePf2+($ac*$K0)*$ePd2)/($ac*$K0)]
set eNd3_forIMK [expr (-$eNf2+($ac*$K0)*$eNd2)/($ac*$K0)]
#puts ePd3_forIMK=$ePd3_forIMK
#puts eNd3_forIMK=$eNd3_forIMK
 ########################################################################################################
 ########################################################################################################
set degflag 0
switch [string tolower $material] {
		pinch4_mod_mat {
     puts "pinching4 --> pinch4_MAT (Gravity frame B)"
     if {$degflag==0} {
 
		# puts ePf3=$ePf3
		# puts ePd3=$ePd3 
		# puts eNf3=$eNf3
		# puts eNd3=$eNd3 
		# puts ePf4=$ePf4
		# puts ePd4=$ePd4 
		# puts eNf4=$eNf4
		# puts eNd4=$eNd4 
 
		# set uForceP 0.05; 
		# set uForceN 0.05;
		# set dmgType "energy"; 
		# set gK1 0; set gK2 0; set gK3 0; set gK4 0; set gKLim 0; 
		# set gD1 0; set gD2 0; set gD3 0; set gD4 0; set gDLim 0; 
		# set gF1 0; set gF2 0; set gF3 0; set gF4 0; set gFLim 0; 
		# set gE 10; 
		


uniaxialMaterial Pinching4 [expr $i*10] $ePf1 $ePd1 $ePf2 $ePd2 $ePf3 $ePd3 $ePf4 $ePd4 $eNf1 $eNd1 $eNf2 $eNd2 $eNf3 $eNd3 $eNf4 $eNd4 $rDispP $fForceP $uForceP $rDispN $fForceN $uForceN $gK1 $gK2 $gK3 $gK4 $gKLim $gD1 $gD2 $gD3 $gD4 $gDLim $gF1 $gF2 $gF3 $gF4 $gFLim $gE $dmgType;
     } else {
       uniaxialMaterial Pinching4 [expr $i*10] \
                        [expr  $ai*0.25]  0.005  [expr  $ai*1.0]  0.025  [expr  $ai*0.55]  0.04  [expr  $ai*0.55]  0.12\
                        [expr -$ai*0.25] -0.005  [expr -$ai*1.0] -0.025  [expr -$ai*0.55] -0.04  [expr -$ai*0.55] -0.12\
                                 0.5 0.25 0.05 \
				 0.5 0.25 0.05 \
                                 0. 0.1 0. 0. 0.2\
				 0. 0.1 0. 0. 0.2\
                                 0. 0.4 0. 0.4 0.9\
				 10. "energy"

     }
     #uniaxialMaterial MinMax $i [expr $i*10] -min -0.12 -max 0.12

uniaxialMaterial MinMax $i [expr $i*10] -min $eNd4 -max $ePd4
set ki [expr $ePf1/$ePd1]
 } imk_mod_mat {
# 05/06/17: Setta i parametri per utilizzare IMK modified anzichè il pinching 4
set ki [expr $ePf1/$ePd1]
set as_Plus [expr $ah]	
 #puts K0=$K0
 #puts as_Plus=$as_Plus
set as_Neg [expr $ah]	
 #puts as_Neg=$as_Neg
set My_Plus [expr $ePf1]
 #puts My_Plus=$My_Plus	
set My_Neg [expr $eNf1]
 #puts My_Neg=$My_Neg
# set Lamda_S 1500
# set Lamda_C 1500
# set Lamda_A 1500
# set Lamda_K 1500	
# set c_S 1	
# set c_C 1	
# set c_A 1		
# set c_K 1
set theta_p_Plus [expr $ePd2-$ePd1]	
 #puts theta_p_Plus=$theta_p_Plus
set theta_p_Neg [expr -($eNd2-$eNd1)]	
 #puts theta_p_Neg=$theta_p_Neg
set theta_pc_Plus [expr $ePd3_forIMK-$ePd2]
#puts theta_pc_Plus=$theta_pc_Plus
set theta_pc_Neg [expr -($eNd3_forIMK-$eNd2)]
#puts theta_pc_Neg=$theta_pc_Neg
set Res_Pos [expr $rp]	
#puts Res_Pos=$Res_Pos
set Res_Neg [expr $rp]	
set theta_u_Plus [expr $ePd4]
#puts theta_u_Plus=$theta_u_Plus
set theta_u_Neg [expr -$eNd4]
#puts theta_u_Neg=$theta_u_Neg
# set D_Plus 1
# set D_Neg 1
uniaxialMaterial ModIMKPeakOriented [expr $i*10] $K0 $as_Plus $as_Neg $My_Plus $My_Neg $Lamda_S $Lamda_C $Lamda_A $Lamda_K $c_S $c_C $c_A $c_K $theta_p_Plus $theta_p_Neg $theta_pc_Plus $theta_pc_Neg $Res_Pos $Res_Neg $theta_u_Plus $theta_u_Neg $D_Plus $D_Neg
uniaxialMaterial MinMax $i [expr $i*10] -min -$theta_u_Neg -max $theta_u_Plus
 } clough_mat {
set ki [expr $ePf1/$ePd1]
uniaxialMaterial Clough [expr $i*10] $K0 $ePf1 $eNf1 $ah $rp $ac $ePd2 $eNd2 $Lamda_S $Lamda_C $Lamda_A $Lamda_K $c_S $c_C $c_A $c_K
uniaxialMaterial MinMax $i [expr $i*10] -min $eNd4 -max $ePd4
} eh_mat {
set Hkin $H_kin_per
set Hiso $H_iso_per
set ki [expr $ePf1/$ePd1]
uniaxialMaterial Hardening $i $K0 $ePf1 $Hiso $Hkin
} steel1_mat {
set ki [expr $ePf1/$ePd1]
uniaxialMaterial Steel01 $i $ePf1 $K0 $ah $a1_st $a2_st $a3_st $a4_st
# uniaxialMaterial MinMax $i [expr $i*10] -min $eNd4 -max $ePd4
 } imk_mod_bilin_mat {
set ki [expr $ePf1/$ePd1]
set as_Plus [expr $ah]	
set as_Neg [expr $ah]	
set My_Plus [expr $ePf1]
set My_Neg [expr $eNf1]
set theta_p_Plus [expr $ePd2-$ePd1]	
set theta_p_Neg [expr -($eNd2-$eNd1)]	
set theta_pc_Plus [expr $ePd3_forIMK-$ePd2]
set theta_pc_Neg [expr -($eNd3_forIMK-$eNd2)]
set Res_Pos [expr $rp]	
set Res_Neg [expr $rp]	
set theta_u_Plus [expr $ePd4]
set theta_u_Neg [expr -$eNd4]

uniaxialMaterial Bilin [expr $i*10] $K0 $as_Plus $as_Neg $My_Plus $My_Neg $Lamda_S_bilin $Lamda_C_bilin $Lamda_A_bilin $Lamda_K_bilin $c_S_bilin $c_C_bilin $c_A_bilin $c_K_bilin $theta_p_Plus $theta_p_Neg $theta_pc_Plus $theta_pc_Neg $Res_Pos $Res_Neg $theta_u_Plus $theta_u_Neg $D_Plus_bilin $D_Neg_bilin $n_bilin
uniaxialMaterial MinMax $i [expr $i*10] -min -$theta_u_Neg -max $theta_u_Plus
 } default {
   error "The material requested is non-existent. Check MatIDA_paramfile.tcl"
   exit
}
}

#-----------------------------------------  
#   if Tperiod <>0 set the mass to achieve the given period.
#   else set the period to match the given mass!
#-----------------------------------------
set kx $ki
if {$Tperiod!=0} {
   # set the mass now. It comes in "tons=1000kg"
   # Remember that T=2*pi*sqrt(m/k) <==> m = (T/(2pi))^2*k
   set mx [expr pow($Tperiod/(2*$pi),2)*$kx]
} else {
   set Tperiod 	[expr 2*$pi*sqrt($mx/$kx)]
}

set omega [expr pow($kx/$mx,0.5)]
puts "initial stiffness K=$kx"
puts "Mass              M=$mx"
puts "SDOF period       T=$Tperiod sec"


# only one material!
set imat_final $i





puts "cyclic frequency  W=$omega rad/sec"
puts "frequency         f=[expr 1.0/$Tperiod] Hz"


node 1 0 
node 2 0  -mass $mx
element zeroLength 1 1 2 -mat $imat_final -dir 1
fix 1 1 
if {$analysistype=="pushover"||$analysistype=="RCycD"} {
   puts "ADDING PARALLEL SPRING FOR PUSHOVER"
   puts "IF NON-CONVERGENCE OCCURS, INCREASE ITS STIFFNESS!"
   uniaxialMaterial Elastic 102 [expr 50*$kx] 0
   node 3 0
   node 4 0
   element zeroLength 2 3 4 -mat 102 -dir 1
   # slave node 4 to node 2
   equalDOF 2 4 1
   fix 3 1
}


recorder Node -file [concat [pwd]/Disp.out] -time -node 2 -dof 1 disp
recorder Node -file [concat [pwd]/Force.out] -time -node 1 -dof 1 reaction

if {!$runIDAflag} {
  recorder Element -file SpringF.out -time -ele 1 force material $imat_final
  recorder Node -file NodeV.out -time -node 2 -dof 1 vel
  recorder Node -file NodeA.out -time -node 2 -dof 1 accel
}




if {$dampingtype=="massproportional"} {
  # values taken from chopra, pg. 418.
  puts "Mass proportional damping [expr $ksi*100]%"
  set alphaM [expr $ksi*2.0*$omega]
  set betaKinit 0
} else {
  puts "Initial stiffness proportional damping [expr $ksi*100]%"
  set alphaM 0
  set betaKinit [expr $ksi*2.0/$omega]
}
# tangent stiffness proportional damping; +beatK*KCurrent
set betaK 0 
# last commited stiffness RAYLEIGH damping parameter; +betaKcomm*KlastCommitt
set betaKcomm 0

rayleigh $alphaM $betaK $betaKinit $betaKcomm

puts "rayleigh $alphaM $betaK $betaKinit $betaKcomm"


constraints Plain
numberer RCM
test NormDispIncr 1.0e-8 20
system BandSPD
#system ProfileSPD
algorithm Newton
integrator Newmark 0.5 0.25

puts [pwd]

# Time to run some timehistory analysis.
if {$analysistype=="dynamic"} {
	source [concat [pwd]/util/DefineXYZ_UniformExcitation.tcl]
	# you are getting back "nptsx" and "dtx"
	DefineXYZ_UniformExcitation $dtsub $g $GMdir $GMfileX $GMfileY $GMfileZ $GMfactorX $GMfactorY $GMfactorZ nptsx dtx
	recorder Node -file NodeAbs.out -timeSeries 1 -time -node 2 -dof 1 accel
	# Common running parameters for both forcing functions
	integrator Newmark 0.5 0.25
	test NormDispIncr 1.0e-8 10 0
	puts "analysis Transient"
	source [concat [pwd]/util/RunTransient2Converge.tcl]
	analysis Transient
	puts nptsx=$nptsx
	# set the number of extra free vibration steps needed after the end of the accelerogram to
	# help converging to a residual displacement
	set Nextrasteps [expr round($Nextracycles*$Tperiod/($dtx/$dtsub))]
	puts "Running additional [expr $Nextracycles*$Tperiod]sec over $Nextrasteps added steps"
	puts dtsub=$dtsub
	set outfile [open [concat [pwd]/Npoints_or.out] "w"]
	puts $outfile $nptsx
	close $outfile	
	RunTransient2Converge [expr $nptsx*$dtsub+$Nextrasteps] [expr $dtx/$dtsub]
	set nptsx [expr $nptsx+$Nextrasteps]
	puts nptsx=$nptsx
    set outfile [open [concat [pwd]/Npoints.out]  "w"]
	puts $outfile $nptsx
	close $outfile	
    #RunTransient2Converge [expr $nptsx*$dtsub] [expr $dtx/$dtsub]
} elseif {$analysistype=="pushover"} {
  source ../util/RunPushover2Converge.tcl
    pattern Plain 2 Linear {
      load 2 1 
    }
  
  set IDctrlNode 2
  
  #
  set Nsteps 700
  set DmaxPush [expr $Nsteps*$DispStep]
  puts "Pushing to [expr $Nsteps*$DispStep]m in $Nsteps steps"
  puts "CAREFUL NEGATIVE SLOPES WILL CAUSE A FAILURE TO CONVERGE"
  integrator DisplacementControl $IDctrlNode 1 $DispStep
  
  test NormDispIncr 1.0e-8 50 0
  analysis Static
  RunPushover2Converge  $IDctrlNode $DmaxPush $Nsteps 1  
  #analyze $Nsteps
  loadConst -time 0.0
} elseif {$analysistype=="RCycD"} {
  source [concat [pwd]/peakpts_.tcl]
  
  # since this is performed under displacement-control, we need a reference load
  pattern Plain 2 Linear {
    load 2 1 
   }
  test NormDispIncr 1.0e-8 20
  algorithm Newton
  source [concat [pwd]/util/RunPushover2Converge.tcl]
  source [concat [pwd]/util/procRCycDAns2.tcl]
  puts "Loading Protocol: $peakpts"
  puts "Performing Reverse Cyclic Displacement Analysis"
  set increments 100
  set nodeTag 2
  set dofTag 1
  ## start procedure for feeding in Reverse Cyclic loading to the model by Disp. control
  procRCycDAns2 $increments $nodeTag $dofTag $peakpts
} elseif {$analysistype=="eigenvalue"} {
  set outfile [open "SDOFModelPeriods.out" "w"]
  puts $outfile $Tperiod
  close $outfile
} elseif {$analysistype=="D2SPO"} {

  source C:/Users/qwerty/Dropbox/me/Vamvatsikos_SDOF_dyn_spo/osees/util/DefineXYZ_UniformExcitation.tcl
  # you are getting back "nptsx" and "dtx"
  DefineXYZ_UniformExcitation $dtsub $g $GMdir $GMfileX $GMfileY $GMfileZ $GMfactorX $GMfactorY $GMfactorZ nptsx dtx
  recorder Node -file NodeAbs.out -timeSeries 1 -time -node 2 -dof 1 accel
  # Common running parameters for both forcing functions
  integrator Newmark 0.5 0.25
  test NormDispIncr 1.0e-8 10 0
  puts "analysis Transient"
  source ../util/RunTransient2Converge.tcl
  analysis Transient
   
   puts nptsx=$nptsx
   puts dtsub=$dtsub
	set outfile [open "Npoints.out"  "w"]
    puts $outfile $nptsx
    close $outfile	
  RunTransient2Converge [expr $nptsx*$dtsub] [expr $dtx/$dtsub]
   # loadConst -time 0.0
	

   puts "ADDING PARALLEL SPRING FOR PUSHOVER"
   puts "IF NON-CONVERGENCE OCCURS, INCREASE ITS STIFFNESS!"
   uniaxialMaterial Elastic 102 [expr 50*$kx] 0.0
   node 3 0
   node 4 0
   element zeroLength 2 3 4 -mat 102 -dir 1
   # slave node 4 to node 2
   equalDOF 2 4 1 
   fix 3 1
	

	  # Fai pushover dopo analisi dinamica
   source ../util/RunPushover2Converge.tcl
    pattern Plain 2 Linear {
      load 2 1 
    }
  
  set IDctrlNode 2
  
  #
  set Nsteps 1000
  set DmaxPush [expr $Nsteps*$DispStep]
  puts "Pushing to [expr $Nsteps*$DispStep]m in $Nsteps steps"
  puts "CAREFUL NEGATIVE SLOPES WILL CAUSE A FAILURE TO CONVERGE"
   
  integrator DisplacementControl $IDctrlNode 1 -$DispStep 
  #loadConst -time 0.0 
  
  test NormDispIncr 1.0e-8 70 0
  analysis Static
  RunPushover2Converge  $IDctrlNode $DmaxPush $Nsteps 1 1 
  # analyze $Nsteps
  loadConst -time 0.0
} elseif {$analysistype=="D2IDA"} {

    # ANALISI DINAMICA
	# you are getting back "nptsx" and "dtx"
	  source [concat [pwd]/MatDYN_thfile.tcl]
	  source [concat [pwd]/util/DefineXYZ_UniformExcitation_DYNONLY.tcl]

      # you are getting back "nptsx" and "dtx"
    DefineXYZ_UniformExcitation_DYNONLY $dtsub $g $GMdir_dyn $GMfileX_dyn $GMfileY_dyn $GMfileZ_dyn $GMfactorX_dyn $GMfactorY_dyn $GMfactorZ_dyn nptsx2 dtx2
	#recorder Node -file NodeAbs.out -timeSeries 1 -time -node 2 -dof 1 accel
	# Common running parameters for both forcing functions
	integrator Newmark 0.5 0.25
	test NormDispIncr 1.0e-8 10 0
	puts "analysis Transient"
	source [concat [pwd]/util/RunTransient2Converge.tcl]
	analysis Transient
	  # set the number of extra free vibration steps needed after the end of the accelerogram to
	  # help converging to a residual displacement
	  set Nextrasteps [expr round($Nextracycles*$Tperiod/($dtx2/$dtsub))]
	  puts "Running additional [expr $Nextracycles*$Tperiod]sec over $Nextrasteps added steps"
	# puts nptsx=$nptsx
	# puts dtsub=$dtsub
	set outfile [open [concat [pwd]/Npoints_b2b_dyn_or.out]  "w"]
	puts $outfile $nptsx2
	close $outfile	
	RunTransient2Converge [expr $nptsx2*$dtsub+$Nextrasteps] [expr $dtx2/$dtsub]
	set nptsx2 [expr $nptsx2+$Nextrasteps]
	
	set outfile [open [concat [pwd]/Npoints_b2b.out]  "w"]
	puts $outfile $nptsx2
	close $outfile	
	 loadConst -time 0.0
	
	    # ANALISI DINAMICA INCREMENTALE
	puts "run IDA"
    source ../util/DefineXYZ_UniformExcitation.tcl	
	# you are getting back "nptsx" and "dtx"
	DefineXYZ_UniformExcitation $dtsub $g $GMdir $GMfileX $GMfileY $GMfileZ $GMfactorX $GMfactorY $GMfactorZ nptsx dtx
	recorder Node -file NodeAbs.out -timeSeries 1 -time -node 2 -dof 1 accel
	# Common running parameters for both forcing functions
	integrator Newmark 0.5 0.25
	test NormDispIncr 1.0e-8 10 0
	puts "analysis Transient"
	# set the number of extra free vibration steps needed after the end of the accelerogram to
	# help converging to a residual displacement
	set Nextrasteps_B2B [expr round($Nextracycles*$Tperiod/($dtx/$dtsub))]
	puts "Running additional [expr $Nextracycles*$Tperiod]sec over $Nextrasteps added steps"
	puts dtsub=$dtsub
	set outfile [open [concat [pwd]/Npoints_or_b2b.out]  "w"]
	puts $outfile $nptsx
	close $outfile	
	source [concat [pwd]/util/RunTransient2Converge.tcl]
	analysis Transient

	puts nptsx=$nptsx
	puts dtsub=$dtsub
	# set outfile [open "Npoints.out"  "w"]
	# puts $outfile $nptsx
	# close $outfile	
	RunTransient2Converge [expr $nptsx*$dtsub+$Nextrasteps_B2B] [expr $dtx/$dtsub]
	loadConst -time 0.0
    set nptsx [expr $nptsx+$Nextrasteps_B2B]
	puts nptsx=$nptsx
	set outfile [open [concat [pwd]/Npoints_ida_b2b.out]  "w"]
	puts $outfile $nptsx
	close $outfile	
}



#if {!$runIDAflag} {
  file delete SDOFModelPrint.out
  print SDOFModelPrint.out
#}
