function [drift,NC,transout] = anlsproc_oseesNDi_IDA_2dir(Sa,runs,thfiles,comp,...
                                                 rsax,sf_yz,recnum,totalnum,...
                                                 anlsdir,exefile,infile,outdir,...
                                                 blackbox_postproc,postproc_input,flag_sc,check_no_sc,...
                                                 runID,spawn)
% An improved version of anlsproc_osees3Dbridge that can handle either 1,2 or 3 components
% of ground motion (2 horizontals + 1 vertical). Completely supercedes older version.
%
% It has the ability to save between IDAs and recover automatically.
% NOTE: it searches for the savefilename and the temporary savefilename
% in the current directory, not in the analysis directory!!
% 
% NOTE: This is a generic procedure that can be used with any file, 2D, 3D, bridge or
% structure. Just change "postproc" to handle the postprocessing right!
%
%
% Example: 
% anlsproc_oseesNDi(2,7,{'../records/BLD090.AT2','../records/BLD360.AT2'},[1,1],1.7,[1],1,10,...
%                     '\users\divamva\opensees\divamva\G7','GenerateG7bridge.tcl','.',...
%                     'postproc_osees3Di_bridge.m',0)
% To use this with traceIDA (or traceDPO4, they are the same) see the file
% runIDA_ND.m
%____________________________________________________________________________________
% OUTPUT
%   NC=non-convergence. 
% INPUT
%   Sa : targetSa to scale to
%   thfiles : cell array with full filenames (including path) of the records to run (1-3).
%   comp    : vector 1x3 with  one "3", the rest "1" or "0". It indicates 
%             what component the XYZ correspond to and which component will be 
%             used for the direct scaling  For example, [3,0,1] 
%             indicates X and Z components supplied, and "Sa" will be
%             scaling of X. It is always suggested to scale by Sa(T1), (minimum 
%             dispersion, so better performance of hunt&fill)
%             so you had better choose this "component-to-scale" as the one
%             the T1 period mostly corresponds to.
%   rsax : unscaled dominant-axis record's sa-value at 1st mode (and make sure this 
%          1st mode actually happens along that axis). Actually it is applied
%          to the first non-zero component of "comp".
%   sf_yz : An extra constant scale factor applied to y or z component (if any).
%         Actually this is applied to the non-dominant components (i.e. if
%         you supply [1,3,1] then sf_yz goes to x and z component. If you
%         supply [1,3,0] then sf_yz had better have only one value,
%         otherwise you will cause problems
%         Usually you had better leave it at [1,1] (two non-dominant components)
%         or [1] (single non-domionant comp) so that all components are scaled by the 
%         same scale factor. Use [1,2] if for example you want the 2nd
%         non-dominant-component to have double scaling factor than the others.
%   anlsdir = directory where both opensees and the infile exist and the analysis
%         takes place.
%   exefile = name (relative to anlsdir) or full path to the opensees
%             executable to be used
%   infile = input filename
%   outdir = directory where the output goes.
%   blackbox_postproc = postprocessing procedure name
%   postproc_input = any input that needs to be passed on to "postproc" procedure
%            should be included here as a single cell-array
%   runID = CPU number (default = NaN)
%   spawn = if 0 then do the normal, running of a single analysis, waiting for it 
%           to finish and then postprocess.
%           if 1 then spawn a separate process for OpenSEES and
%           return control back to the calling routine. Postprocessing should be
%           handled there by calling this routine again with spawn=2.
%______________________________________________________________________________________
% Created 21/Jan/2005, by D.Vamvatsikos
% Updated 5/Oct/2005:  Added the option anls.exefile
% Updated 17/May/2009: Added "spawn", i.e. the option to spawn separate
%          processes for OpenSees used parallel IDA running. This
%          necessitates any kind of postprocessing calls 
%          to be handled by calling this routine a second time.
% Updated 17/Dec/2011: Added double quotes to enclose filenames and
%          recordnames sent to the system or to Opensees to make sure that
%          directories with spaces in their name can be used.
%
if nargin<18, spawn=0; end
if nargin<17, runID=NaN; end
currentdir=cd;
cd(anlsdir);
% better not to use tic/toc as they use global variables and will fail
% if another procedure calls tic internally
persistent t0


if spawn~=2
   t0=clock;
   %--------------------------------------------------------------
   % Generate the tcl-file to define record name and scale factor
   %--------------------------------------------------------------
   ncomp=length(thfiles);
   % calculate the scaling factor (for dominant-axis component)
   % If other components exist, apply the differential scaling factors that
   % the user may have requested (if any).

   ithree=find(comp==3);
   if flag_sc==0
    sf(ithree)=Sa/rsax;
   else
       % Questo serve per differenziare caso Dyn da caso MSA. In
       % quest'ultimo infatti non applico direttamente un fattore di scala
       % bens scalo il record in modo tale da avere una determinata Sa
       if Sa==0 || Sa==0.01 % perch se Sa==0 automaticamente l'algoritmo la porta a 0.01
            sf(ithree)=flag_sc;
       else
             if check_no_sc==0
                sf(ithree)=Sa/rsax;
             else
                sf(ithree)=1;
             end
       end
   end
   izero=find(comp==0);
   if ~isempty(izero)
      sf(izero)=0;
      % note that Tcl will correctly interpret <set GMfileZ ""> to make the variable
      % empty. Then checking <if {$GMfileZ==""} { .... }> will reveal that
      % no component exists in that direction!!
      % Note that since by default now the xyz_thfiles{..} is surrounded by
      % "", we can just make it empty now, i.e. '', otherwise you get """"
      % in Opensees and it complains! 
      %[xyz_thfiles{izero}]=deal('""');
      [xyz_thfiles{izero}]=deal('');
   end
   ione=find(comp==1);
   if ~isempty(ione)
      sf(ione)=sf(ithree).*sf_yz;
   end
   % thfiles may only have one or two filenames, sequentially corresponding to
   % the non-zero components. So make the proper transfer here! The zeroed
   % components have already gotten "" as filename!
   xyz_thfiles(sort([ithree,ione]))=thfiles;

   % create the timehistory information file for OpenSees to read.
   while 1
      fout=fopen([pwd,'\','MatIDA_thfile.tcl'],'w');
      % this has managed to fail twice in parallel processing. I am not sure why.
      % It seems that the file cannot open, while if you try a bit later it will.
      % Some kind of windows file management issue I guess.
      if fout~=-1,
         break
      else
         fprintf('Cannot open MatIDA_thfile.tcl. Retrying! \n')
         % Wait a bit.
         pause(0.05);
      end
   end
   fprintf(fout,'set GMdir ""\n');
   % Enclose filenames in quotes to avoid problems with any spaces included
   % in directory names. 
   fprintf(fout,'set GMfileX "%s"\n',xyz_thfiles{1});
   fprintf(fout,'set GMfileY "%s"\n',xyz_thfiles{2});
   fprintf(fout,'set GMfileZ "%s"\n',xyz_thfiles{3});
   fprintf(fout,'set GMfactorX %g\n',sf(1));
   fprintf(fout,'set GMfactorY %g\n',sf(2));
   fprintf(fout,'set GMfactorZ %g\n',sf(3));
   fclose(fout);

   %--------------------------------------------
   % Run OpenSEES
   %--------------------------------------------
   %eval(['!openSees ' infile '>screendump']);
   % Note that you need the "" in general. If path information is included
   % especially something like "..", then simply >> ! ../opensees << will not
   % work! On the other hand >> ! "../opensees" always works!
   if isnan(runID)
      % the old style call, non-parallel version
      % The double quotes also allow spaces in directory names.
      eval(sprintf('!"%s" "%s" >screendump',exefile,infile));
   elseif spawn==0
      % new style: runs in a minimized window, easier to kill, easier to track
      % down by name. Delete ncerror.out first to make sure that if opensees
      % exits without creating it, then it did not finish properly, i.e. it
      % was killed by a slave-1 procedure (or the code crashed! Careful)
      delete('ncerror.out')
      [s,w]=system(sprintf('start "OSEES-%s" /wait /min %s %s',runID,exefile,infile));
   else
      delete('ncerror.out')
      [s,w]=system(sprintf('start "OSEES-%s" /min %s %s',runID,exefile,infile));
      NC=[];
      transout={{}};
      drift=[];
      return
      % ACTUALLY WE SHOULD EXIT RIGHT NOW!!!
      % Go back to traceIDAp to give it control. It should 
      % keep checking with us and return control for postprocessing.
   end
end
%--------------------------------------------
% PostProcessing the output
% If you set spawn=2, only this part will run.
%--------------------------------------------
% gather all drifts
cd(outdir);

% Opensees will create a file called "ncerror.out" and write the value 
% inside (1 or 0) to denote whether an error caused the exit (i.e. a
% collapse occured). This is how I check for collapse  
exout=exist('ncerror.out','file');
if exout==0||dlmread('ncerror.out')~=0
   NC=1;
   transout={{}};
   drift=NaN;
else
   NC=0;
   % call the specific postprocessing routine whose name was passed on!
   [drift,tout]=feval(blackbox_postproc,postproc_input{:});
   transout={tout};
   transout{1,1}.sf=sf(1:2);
end

% this is more robust than tic/toc
if exout==0, fprintf('***** Run Cancelled\n'); end
fprintf('Elapsed time %4.2f min\n',etime(clock,t0)/60) 
fprintf('Record %d/%d (run %d): SA= %0.3gg, drift= %0.3g\n',...
         recnum,totalnum,runs+1,Sa,drift)
cd(currentdir);