function [sa,sv,sd,ut,ut_dot,ut_ddot]=linearSDOF_nthstory(T,ksi,...
                                            ugt_ddot,gdt,dt,u0,u0_dot)
%_________________________________________________________________________
% INPUT
%    T    : vector of periods (sec)
%    ksi  : critical damping ratio (e.g. 0.05)
%  ug_ddot: ug_double-dot = ground acceleration
%    gdt  : ground motion time step (sec)
%    dt   : analysis time step (sec). Default is equal to gdt.
%    u0   : initial displacement (default 0)
%   u0_dot: initial velocity (default 0)
%
% OUTPUT
%   sa    : Sa, one for each period.
%   sv    : Sv, one for each period.
%   sd    : Sd, one for each period.
%   ut, ut_dot, ut_ddot: vectors for disp/vel/accel. For multiple periods
%           only the vectors for the last one are returned
%_______________________________________________________________________
%
% Created 21/Sep/2006, Copyright (c) 2006 by D. Vamvatsikos
% Updated 22/Nov/2011: Removed mass as an input to simplify everything. 
%         The normalized equations are essentially solved, where only damping 
%         and period matter. Various other numerical tricks have been
%         implemented for reasons of efficiency. This makes it essentially
%         a competitive algorithm to replace SNAP for all my operations. 
%         It can also handle a vector of periods now.


% Example (from Kiremidjan's notes)
%  ugt_ddot=[0,5,20,45,45,30,0,-30,-45,-45,-20,-5,zeros(1,40)];
%  ksi=0.1581; T=0.9935; gdt=0.1; dt=0.025
% [sa,sv,sd,ut,ut_dot,ut_ddot]=linearSDOF_thstory(T,ksi,M,ugt_ddot,gdt,dt);
% 
% CAREFUL: Careful with the units in ugt_ddot. If in g's, and dt in secs then velocity
% will be g*sec and displacement g*sec^2.
%
% The results agree completely with SNAP by the way and this is indeed
% faster on a Pentium IV 800Mhz by 0.1-0.2sec. Not bad.

% default is zero speed and displacement at start
if nargin<7, u0_dot=0; end
if nargin<6, u0=0; end
if nargin<5, dt=gdt; end
if nargin<4, error('need at least 4 inputs'); end


% interolate the ground motion for the new dt supplied
if dt~=gdt
   N1=length(ugt_ddot);
   t1=linspace(0,(N1-1)*gdt,N1);
   N2=ceil((N1-1)*gdt/dt)+1;
   t2=linspace(0,(N2-1)*dt,N2);
   % use linear interpolation with 0's when extrapolation is requested (e.g.
   % if gdt/dt is not an integer
   ugt_ddot=interp1(t1,ugt_ddot,t2,'linear',0);
end

ut=zeros(1,length(ugt_ddot));
ut_dot=ut;
ut_ddot=ut;

ut(1)=u0;
ut_dot(1)=u0_dot;

% I have set M=1 everywhere below. 
% T=2*pi*sqrt(m/k) ==> k=M*w0^2
w0=2*pi./T; % rad/sec
Kt=w0.^2;
%f0=1./T; % Hz
C=2*ksi*w0; % damping coefficient
% Mass proportional damping
a=2*w0*ksi;
% To avoid dividing by dt all the time:
invdt=1/dt;
invdt2=invdt.^2;

for j=1:length(T)
   Ftdt=-ugt_ddot(1);
   Rtdt=Kt(j)*ut(1); % restoring force for elastic spring
   % to satisfy equilibrium at t=0. IS THIS CORRECT for non-zero initial conditions?
   % Should be!
   ut_ddot(1)=Ftdt-C(j)*ut_dot(1)-Rtdt;
   % Newmark's average acceleration method
   for i=1:(length(ugt_ddot)-1)
      At=-(4*invdt)*ut_dot(i)-ut_ddot(i);
      Bt=-ut_dot(i);
      Ktbar=Kt(j)+2*a(j)*invdt+4*invdt2;
      Rt=Rtdt; % previous restoring force
      Ftdt=-ugt_ddot(i+1);
      DRtbar=Ftdt - Rt  - (At+a(j)*Bt);
      Dut=DRtbar/Ktbar;
      Dut_dot=2*Dut*invdt-2*ut_dot(i);
      ut(i+1)=ut(i)+Dut;
      Rtdt=Kt(j)*ut(i+1);% restoring force (spring force) for elastic system)
      ut_dot(i+1)=ut_dot(i)+Dut_dot;
      ut_ddot(i+1)=Ftdt-C(j)*ut_dot(i+1)-Rtdt;
      % This one does not preserve equilibrium at the end of step.
      % But it comes out exactly the same as above for linear elastic system
      %ut_ddot_e(i+1)=4*Dut/dt^2+At;
   end
   sd(j)=max(abs(ut));
   sv(j)=max(abs(ut_dot));
   % ut_ddot is relative acceleration. But Sa is absolute, so add up the
   % ground acceleration.
   sa(j)=max(abs(ut_ddot+ugt_ddot'));
end


%% Plot response timehistories
%    t=linspace(0,(length(ut)-1)*dt,length(ut));
%    subplot(3,1,1)
%    plot(t,ut,'b-'); ylabel('disp'); grid on
%    subplot(3,1,2)
%    plot(t,ut_dot,'b-'); ylabel('vel'); grid on
%    subplot(3,1,3)
%    plot(t,ut_ddot,'b-'); ylabel('accel'); grid on
%    xlabel('time')
%
%% check for errors
%subplot(3,1,1); hold on
%plot(t,ut_ddot_e,'r--')
