function [x,y,ddydx,flag]=dxy2xy(dxy,cv)
% function [x,y,ddydx,flag]=dxy2xy(dxy,cv)
%-----------------------------------------------------------------------
% INPUT
% cv : Natural Spline curve produced thru "cscvn" or similar
%      it is essentialy a piecewise polynomial in pp-form, that
%      uses a parameter t to produce (x,y) values.
%      i.e. cv(t)=(x(t),y(t)) ( R-->R^2 function )
%      what this routine does is take a dy/dx value (i.e. a slope), 
%      find corresponding t and then produce (x,y) value. Since "cv" is 2D 
%      curve, you may get multiple (x,y)'s. Actually it is a way
%      to find where on the curve is the supplied slope!
% dxy: Value of dy/dx slope whose corresponding (x,y) you wish
% OUTPUT
% x,y  : Value(s) of (x,y) corresponding to the slope.Could be a whole vector!
% ddydx: Curvature at the (x,y) points.
% flag : If it is 1 then the specified dx/dy is not producing an acceptable
%        value of "t", i.e the routine will only interpolate(flag=0), but
%        not extrapolate (flag=1). If such an "x,y" is required, an obsene 
%        value of (x,y)=(10,10) will be returned and flag is raised to 1.
%------------------------------------------------------------------------
% CAUTION :
% Due to the nature of the spline approximation, i.e piecewise cubic
% polynomials, even though macroscopically it is not apparent, you should 
% expect quite a few minor slope variations. 
% e.g. even if you supply a perfect monotonic set of points, the
% approximation will be mostly monotonic, but in some small intervals it
% will wiggle back and forth. So don't trust the results when you supply
% very small values of dxy. Or do trust them, but there are probably going
% to be a lot of them! Of course that is not a problem for us
% as collapse happens at large values of dy/dx (dtheta/dsa). Things look 
% somewhat better there although wiggles still exist.
% (Last Updated: September 1st, 1999)
% Copyright (c) 1999 by D.Vamvatsikos

% tolerance measure. Solely used to include the points close to 
% the knots, that usually may be missed due to round off errors
tol=1E-14;

flag=0; t=[];
dcv=fnder(cv,1);
[brk,C,n,k]=ppbrk(dcv); % get breaks, polynomial coefficients and # of breaks 

for i=1:n
  r=roots(C(i*2,:)-dxy*C(i*2-1,:))+brk(i);% find roots of each polynomial piece
  % now choose only those roots that exist within interval of that pp.
  % the local coords are [0,brk(i+1)-brk(i)] --> domain of def. of i-part.
  % NOT [brk(i),brk(i+1)], but if we add brk(i) to the roots we return
  % to global coords. Well, it t=brk(n) then it will be rejected,
  % but who cares. Not a big problem. Check the transpose needed 
  % for r(find...). That is because if 2 or three of r's components
  % pass the test then you will be trying to concatenate t (row vector)
  % with a column vector (r(find..)), so transpose the column vector!
  t=[t,r(find(r>=brk(i) & r<brk(i+1)+tol & imag(r)==0))'];
end

if isempty(t)
  flag=1; % extrapolation danger. Raise flag.
  y=NaN;x=NaN;
  ddydx=NaN;
else
  t=unique(t);  %// get rid of repetitions that "tol" might include
  xy=fnval(cv,t);  % now evaluate the "cv" curve at found "t"
  y=xy(2,:);  % separate the y's from xy points
  x=xy(1,:);
  dxydt=fnval(dcv,t); % (dx/dt,dy/dt) points
  dxdt=dxydt(1,:);  % dx/dt
  dydt=dxydt(2,:);  % dy/dt
  dydx=dydt./dxdt;  % dy/dx
  ddcv=fnder(dcv,1); % 2nd derivative w.r.t "t"
  ddxydt=fnval(ddcv,t);
  ddxdt=ddxydt(1,:); % ddx/d(t^2)
  ddydt=ddxydt(2,:); % ddy/d(t^2)
  ddydx=(ddydt.*dxdt-ddxdt.*dydt)./(dxdt).^3; % ddy/d(x^2)
  
  %dxydt(2,:)./dxydt(1,:)
  %if any( dxy~=dxydt(2,:)./dxydt(1,:)), disp('Error in dxy2xy.m'), end  
  % all these should be equal. Due to approximations this will cause false 
  % alarms. Use it only for debugging.
end
