function [x,y,ddydx,flag]=sec2xy(x_y,cv)
% function [x,y,ddydx,flag]=sec2xy(x_y,cv)
%-----------------------------------------------------------------------
% INPUT
% cv : Natural Spline curve produced thru "cscvn" or similar
%      it is essentialy a piecewise polynomial in pp-form, that
%      uses a parameter t to produce (x,y) values.
%      i.e. cv(t)=(x(t),y(t)) ( R-->R^2 function )
%      what this routine does is take a secant slope value (i.e y/x value), 
%      find corresponding t and then produce (x,y) value. Since "cv" is 2D 
%      curve, you may get multiple (x,y)'s. Actually it is a way
%      to find where on the curve is the supplied slope!
% x_y: Value of y/x slope whose corresponding (x,y) you wish
% OUTPUT
% x,y  : Value(s) of (x,y) corresponding to the slope.Could be a whole vector!
% ddydx: Curvature at the (x,y) points.
% flag : If it is 1 then the specified dx/dy is not producing an acceptable
%        value of "t", i.e the routine will only interpolate(flag=0), but
%        not extrapolate (flag=1). If such an "x,y" is required, an obsene 
%        value of (x,y)=(10,10) will be returned and flag is raised to 1.
%------------------------------------------------------------------------
% (Last Updated: September 1st, 1999)
% Copyright (c) 1999 by D.Vamvatsikos

% tolerance measure. Solely used to include the points close to 
% the knots, that usually may be missed due to round off errors
tol=1E-14;

flag=0; t=[];
[brk,C,n,k]=ppbrk(cv); % get breaks, polynomial coefficients and # of breaks 

for i=1:n
  r=roots(C(i*2,:)-x_y*C(i*2-1,:))+brk(i);% find roots of each polynomial piece
  % now choose only those roots that exist within interval of that pp.
  % the local coords are [0,brk(i+1)-brk(i)] --> domain of def. of i-part.
  % NOT [brk(i),brk(i+1)], but if we add brk(i) to the roots we return
  % to global coords. From conditions below, t=0 will never be accepted as
  % as solution ( as brk(1)=0), but it is not expected to be a solution!
  % as 0/0=inf
  % Not below the transpose on r(find..). For an explanation
  % check dxy2xy
  t=[t,r(find(r>brk(i) & r<=brk(i+1) & imag(r)==0))'];
end

if isempty(t)
  flag=1; % extrapolation danger. Raise flag.
  y=NaN;x=NaN;
  ddydx=NaN;
else
  t=unique(t);  %// get rid of repetitions that "tol" might include
  xy=fnval(cv,t);  % now evaluate the "cv" curve at found "t"
  y=xy(2,:);  % separate the y's from xy points
  x=xy(1,:);
  % evaluate curvature
  dcv=fnder(cv,1);
  dxydt=fnval(dcv,t); % (dx/dt,dy/dt) points
  dxdt=dxydt(1,:);  % dx/dt
  dydt=dxydt(2,:);  % dy/dt
  dydx=dydt./dxdt;  % dy/dx
  ddcv=fnder(dcv,1); % 2nd derivative w.r.t "t"
  ddxydt=fnval(ddcv,t);
  ddxdt=ddxydt(1,:); % ddx/d(t^2)
  ddydt=ddxydt(2,:); % ddy/d(t^2)
  ddydx=(ddydt.*dxdt-ddxdt.*dydt)./(dxdt).^3; % ddy/d(x^2)

  %if any( x_y~=y./x), disp('Error in sec2xy.m'), end  
  % all these should be equal. Due to approximations this will cause false 
  % alarms. Use it only for debugging.
end
