function [recinfo,Eflag,ug,header]=get_recinfo4(thfilename,rectype,damping,period,...
                                      PSAflag,flag_for_tail)
%-------------------------------------------------------------------------
% INPUT
%  thfilename : the time history file name, that we want to investigate
%               note that it must be the full path and name for us to find
%               it''s location.
% rectype     : the record file name type, for now, only the type 'sac' and
%               'silva' are supported. 'Silva' is same as 'peer'. Both
%               older PEER and PEER NGA records are supported.
% damping     : damping (usually 0.02 or 0.05)
% period      : a vector of periods
% PSAflag     : If 1 (default) then Sa is set equal to PSa. This is the
%               classic setting in SNAP as well. Use 0 to see the
%               difference
% flag_for_tail: 1 to add a tail for 10 seconds havin 0 g accelleration
% OUTPUT
%  recinfo: a structure with fields
%     fname  : record file name (and directory location) of record file
%                 (Any headers are moved to the end always)
%     title  : A description of the record if any is available in the file.
%     period : The structure's 1st period
%     damping: The damping used to calculate "1st mode Sa"
%     units  : multiplicative factor to convert record accel to cm/sec^2
%     sa     : The SA at the supplied periods (set equal to psa for
%             PSAflag=1)
%     psa    : The pseudo Sa at the supplied periods.
%     dt     : the spacing "dt" of the time steps
%     ncol   : the number of columns of acceleration values per row (usually 6)
%     nrow   : the number of lines of acceleration values in the file. 
%     time   : The total record time.
%     type   : denotes type of record (silva, or sac)
%     pga    : PGA of record
%     pgv    : PGV of record (in cm/s)
%  Eflag: if Eflag=1 then an error was encountered, else all is ok.
%     ug: the accelerogram vector in units of g
% header: The header lines for the accelerogram
%-------------------------------------------------------------------------
% NOTE: This function extracts info from an acceleration time history file
% the assumption is that several lines of text exist at the beginning
% or end of the file that contain the required info, while all 
% the rest is columns of acceleration values.
%
% Created: Copyright (c) 1999 by D.Vamvatsikos
% Updated: October 6th 1999
%      Now returns a "recinfo" structure
% Updated: Jan 21, 2005
%      Made compatible with Matlab 6.5 by replacing fline==-1 with 
%      ~ischar(fline) when looking for valid Silva records (about line 120+).
% Updated: Feb 28, 2006
%      Applied the above correction to line ~160. Also made it compatible
%      with PEER NGA records and newer snap_elas_Sa_spect.m to be able to
%      read Pseudo-Sa information as well.
% Updated: Jun 14, 2006
%      recinfo also containts the npts field, i.e. how many points are in
%      the record. Usually npts>=nrow*ncol. The reason is that the last row
%      may not be complete (i.e. have all the columns full) therefore it is
%      discarded from estimating nrow so that older programs, like DRAIN
%      (SNAP?) will not generate an error. Additionally, if snapdir is not
%      supplied, or left empty, then SNAP will not be run!
% Updated: Apr 2, 2008
%      Now the title of the record (event and station) is extracted and
%      added to the recinfo.title field.
% Updated: Feb 3, 2010
%      Added estimation of PGV for ATC76 project purposes
% Updated: Nov 22, 2011
%      Changed SDOF running from SNAP to my Matlab routine.
% Updated: 13/Feb/2012
%      Added commentary on newer NGA files that thankfully are handled
%      properly by this routine without need for modifications!
%      Note that they are flagged as ngaflag==0, as their NPTS/DT header is
%      old-style.
% Updated 20/Oct/2013
%      Now also returns the acceleration time history in units of g.
% Updated 01/Nov/2013
%      It has been made faster by storing previous lines in a stack when
%      scanning the file for the header. It also returns the header if
%      requested (at least for PEER files).
% Updated 12/Mar/2014
%      I fixed a major error introduced in the preceding update, where files 
%      with the header still in the first four lines were not having their 
%      acceleration values read at all. 
% Updated 10/Nov/2014
%      Some PEER NGA records appear with "NPTS=... " in the fourth line, rather
%      than "num,num NPTS, DT". Now these can be read. The only modification was
%      to remove the checking for "PEER NGA" in the first line. The ngaflag was
%      useless after all. 

% Example of use
% NEW version:
% thfile='c:\users\divamva\own\opensees\divamva\records\NGA_no_121_A-BCS000.AT2';
% thfile='c:\users\divamva\own\opensees\divamva\records\AGW000.AT2';
% thfile='C:\Users\divamva\own\PhDs-CoOps\baltzopoulos\NGA_no_28_C12050_WORKS.AT2';
% thfile='C:\Users\divamva\own\PhDs-CoOps\baltzopoulos\RSN159_IMPVALL_H_H-AGR273_DOESNT.AT2';

% tic;[recinfo,Eflag]=get_recinfo4(thfile,'silva',0.05,[0.1,0.5,1,2]),toc
% OLD version:
% tic;[recinfo,Eflag]=get_recinfo3(thfile,'silva',0.05,[0.1,0.5,1,2],get_snapdir),toc
%
% NOTE: you will see differences only at very low periods, around 0.01sec
% between SNAP and my Matlab routine. Round-off errors start piling up
% around there, although dt is usually quite lower (0.005 is typical). Remember
% that the stability limit for linear acceleration is 0.551*Tperiod.

%keyboard
if nargin<6, flag_for_tail=0; end
if nargin<5, PSAflag=1; end
if nargin<4, period=[]; damping=[]; end
if nargin<2, error('need at least 2 inputs'); end

acc=[];

fid=fopen(thfilename,'r');
if fid==-1, Eflag=1; else Eflag=0; end;
switch lower(rectype)

  case 'sac', 
% below follows what a header (or tailer) of SAC record looks like.
% look after the %//. Sometimes a '%' precedes or not these lines.
%-----------------------------------------------------------------
%//% -0.49641E-03  0.49879E-03 -0.54021E-03  0.49490E-03
%//%  Time his fp  matched to spectrum:Sclrel_output/lgpc.h.freq.tgt 
%//%   2500    1.00000E-02  0  0  0.  0.  0.  0.  0.
%-----------------------------------------------------------------
% only the SE records contain only numbers. For those it is rather easy.
% just count the numbers!
    while 1
      fline = fgetl(fid);
      % check where the character values begin.
      % after the first pure text (descriptive) line the first numerical
      % line contains total # of points and the dt (in that order)
      if isempty(str2num(strtok(fline)))   
         % non-numerical value, set this line as title and then break.
         rectitle=fline;
         break;
      else
         [accline,count,errmsg]=sscanf(fline,'%f');
         acc=[acc;accline];
      end
    end    
    % now look for the first numerical-value!
    while 1 
      fline = fgetl(fid); 
      tok1=get_nth_tok(fline,1);
      tok2=get_nth_tok(fline,2);
      if (~isempty(str2num(tok1)))  % found 1st number, no '%' preceding
         ntot = str2num(tok1);
         dt = str2num(tok2);
         break;
      else 
        if ((~isempty(str2num(tok2)))&(tok1=='%'))
          ntot = str2num(tok2);
          dt = str2num(get_nth_tok(fline,3));
          break;
        end
      end
    end
    if (isempty(dt)) Eflag=1; end
    ncol = 6; %// standard for all SAC records

    %// for DRAIN to accept it, nrow*ncol=ntot. So discard non-full rows.
    nrow = (ntot - mod(ntot,ncol))/ncol;
    %// all SAC records come in cm/s2
    units=1;
    fclose(fid);
    header={};

    
    
  case {'silva','peer'}
% below follows what a header (or tail) of Silva record looks like
%----------------------------------------------------
%PACIFIC ENGINEERING AND ANALYSIS STRONG-MOTION DATA
%LOMA PRIETA 10/18/89 00:05, HOLLISTER DIFF ARRAY, 165
%ACCELERATION TIME HISTORY IN UNITS OF G
%NPTS=  7928, DT= .00500 SEC
%-----------------------------------------------------
%      The header in the PEER NGA record is formatted as follows:
%------------------------------------------------------
%PEER NGA STRONG MOTION DATABASE RECORD
%CORINTH EQ, 02/24/81, M6.7, CORINTH, TRANSVERSE                                 
%ACCELERATION TIME HISTORY IN UNITS OF G
%4094    0.0100    NPTS, DT
%------------------------------------------------
% OR
%------------------------------------------------
%   PEER NGA STRONG MOTION DATABASE RECORD
%   FRIULI, ITALY 05/06/76 2000, BARCIS, 000                                        
%   ACCELERATION TIME HISTORY IN UNITS OF G
%   NPTS=3309,    DT=0.0050SEC
%-------------------------------------------------
    % if this is a header move it to the tail, otherwise neither snap, nor
    % drain2dx, nor snap2dx will be able to read it
    % NOTE: We could use "m" to get the number of rows, instead 
    % of reading NPTS, but we would never be able to figure out 
    % "DT". So we need the header.
        
    m=0;   % a line counter
    ngaflag=0;
    fline=cell(4,1);
    headerfound=0;
    while 1
      m=m+1;
      % keep 4 lines, the current plus three past. The current minus 2 positions
      % containts the title when you hit the NPTS line.
      
      fline=fgetl(fid);
      % ofline works like a stack of size 4
      k=mod(m-1,4)+1;
      ofline{k}=fline;
      
      % find out if this is a PEER NGA file. Do this by looking for PEER NGA
      % at the start of any line. For speed, perform
      % this comparison only if the ngaflag is still zero!
      
      % get to the point where they start providing NPTS! 
      if ~headerfound
         if strncmp(strtok(fline),'NPTS=',5)
            [ntot,dt]=strread(fline,'NPTS= %d , DT= %f',1);
            rectitle=ofline{mod(m-3,4)+1};
            % renumber to get the right order for the header.
            %header=ofline(mod(m-[4,3,2,1],4)+1);
            headerfound=1;
            headerpos=m;
            continue
         elseif strfind(fline,'NPTS')>5
            % "NPTS" must appear beyond the first 5 characters
            [ntot,dt]=strread(fline,'%d %f',1); %use pattern only once, otherwise it fails!
            rectitle=ofline{mod(m-3,4)+1};
            % renumber to get the right order for the header.
            %header=ofline(mod(m-[4,3,2,1],4)+1);
            headerfound=1;
            headerpos=m;
            continue
         end
      end 
         
      if ~ischar(fline)   % this is a nice way to check for end-of-file
         if ~headerfound
            Eflag=1;
            pwd_opening=getappdata(0,'pwd_opening');
            if isempty(pwd_opening)==0
                cd(pwd_opening)
            end
            errordlg('Not a valid PEER/Silva record','Error')
            error('Not a valid PEER/Silva record')
         else
            break
         end
      else
         [accline,count,errmsg]=sscanf(fline,'%f');
        
         if ~isempty(accline) % make sure it contains numbers!
            acc=[acc;accline];
         end
      end
    end
    % 09/01/18 - Set the problem borning when the record title starts with
    % a number
    if length(acc)>ntot
        if headerpos<=4
            acc(1)=[];
        else
            acc(end)=[]; 
        end
    end
    if isempty(dt)||isempty(ntot), Eflag=1; end
    ncol=5; % standard Silva 5-columns!
    units=981; %(convert from g's to cm/s2)
    nrow = (ntot - mod(ntot,ncol))/ncol;
    fclose(fid);
    % For files that have the text info as header, move it to 
    % the back
    if headerpos<=4  
       fprintf('\n Virgin Rec, moving header to the tail\n')
       fid=fopen(thfilename,'r');
       for i=1:headerpos
          % I know I am reading it again, but I need to do it anyway to discard
          % the first four lines of the header and go to accel values.
          header{i}=fgets(fid);
       end
       tmp1=tempname;
       fout=fopen(tmp1,'w');
       while 1
          fline = fgets(fid);
          if ~ischar(fline), break; end
          fprintf(fout,'%s',fline);
       end
       % put the header at the end
       for i=1:headerpos
          fprintf(fout,'%s',header{i});
       end
       fclose(fout)
       fclose(fid)
       % remove original file and replace it with the new one
       copyfile(tmp1,thfilename);
       delete(tmp1);
    end
    
%     if flag_for_tail==1
%         % Add tail in PEER record!
%         time_tail=20;
%         num_od_zero=round(time_tail/dt);
%         ntot=ntot+num_od_zero;
%         acc=[acc;ones(num_od_zero,1)*0.00001];
%         tmp1=tempname;
%         fid=fopen(thfilename,'r');
%         fout=fopen(tmp1,'w');
%         fprintf(fout,'%15.6e %15.6e %15.6e %15.6e %15.6e\n',acc);
%         rec_header1 = 'PEER NGA STRONG MOTION DATABASE RECORD';
%         rec_header2 =rectitle;
%         rec_header3 =  'ACCELERATION TIME HISTORY IN UNITS OF G';
%         rec_header4 = strcat(num2str(ntot),{'    '},num2str(dt,'%.4f'),{'    NPTS, DT'});
%         fprintf(fout,'%s\r',rec_header1,rec_header2,rec_header3,rec_header4{1});
%         fclose(fout);   
%         fclose(fid);
%         copyfile(tmp1,thfilename);
%         delete(tmp1);
%     end
otherwise
   error('get_rec_info() : Unsupported Record Type')
end


%// prepare output 
recinfo.fname = thfilename;
recinfo.title = rectitle;
recinfo.damping = damping;
recinfo.period = period;   
recinfo.units = units;
recinfo.nrow = nrow;
recinfo.ncol = ncol;
recinfo.npts = ntot;
recinfo.dt   = dt;
recinfo.time = (nrow*ncol-1)*dt; 
recinfo.type = rectype;
recinfo.acc=acc;
if length(recinfo.acc)>recinfo.npts
recinfo.acc(end)=[];
end
t=linspace(0,(ntot-1)*dt,ntot);
if  length(acc)<length(t);
    acc=[acc' acc(end)]';
end
if length(acc)~=length(t)
acc=acc(1:length(t));
end
recinfo.acc=acc;
recinfo.pgv = max(abs(cumtrapz(t,acc*units))); % comes out in cm/s
recinfo.pga = max(abs(acc*units))/981; % To come out in g's

if ~isempty(period)
   % acc*units is always in cm/sec2, dt is in sec, thus
   % sa is in cm/sec2, sv in cm/sec, sd in cm (always)
   %[sa,sv,sd,ut,ut_dot,ut_ddot]=linearSDOF_thstory(period,damping,1,acc*units,dt);
%    [sa,~,sd,~,~,~]=linearSDOF_nthstory(period,damping,acc*units,dt);
   [sa,~,sd]=spectra_light(dt,acc,damping*100,1,period);
   sd=sd*981; %cm
   % Actually, all the time we use PSa. So, since SNAP does the same and all
   % my routines use recinfo.sa to do their work, the default is to let Sa be
   % equal to PSa. Thus this remains compatible with older versions.
   recinfo.psa=(sd*4.*(pi./period).^2)/981;
   if PSAflag
      recinfo.sa=recinfo.psa;
   else
      recinfo.sa = sa/981;
   end
else
   recinfo.sa=[]; recinfo.pga=[]; recinfo.psa=[];
end
recinfo.sa
if nargout>2
  ug=acc*(units/981);
end
